@@ -89,9 +89,10 @@ check_args_coef <- function(rho){
 
 #TODO
 check_args_constraints <- function(rho){
-  if (!all(rho$coef.names %in% names(rho$constraints))) stop("coef.constraints need to be specified for all covariates.", call. = FALSE)
+  if (!all(rho$coef.names %in% names(rho$constraints)))
+    stop("coef.constraints need to be specified for all covariates.", call. = FALSE)
   #check dimensions of rho$constraints
-  if(any(sapply(rho$constraints, NROW) != rho$ncats)) stop("coef.constraints need to have number of total categories rows
+  if(any(sapply(rho$constraints, NROW) != rho$nthetas)) stop("coef.constraints need to have number of total categories rows
                                                            (sum of number of categories for all dimensions).", call. = FALSE)
 
 
@@ -111,26 +112,29 @@ get_start_values <- function(rho){
 c(unlist(gammas), rep(0, rho$npar.betas))
 }
 
-
+## par <- rho$optpar
 transf_par <- function(par, rho) {
   tparsigma <- par[rho$npar.thetas + rho$npar.betas +
     seq_len(attr(rho$error.structure, "npar"))]
   sigmas <- build_error_struct(rho$error.structure, tparsigma)
   theta <- rho$transf_thresholds(par[seq_len(rho$npar.thetas)], rho)
   par_beta <- par[rho$npar.thetas + seq_len(rho$npar.betas)]
-
-  beta <- lapply(seq_len(rho$ndim), function(j){
-    sapply(seq_along(rho$coef.names), function(p) {
-      ifelse(is.na(rho$coef.ind[[j]][,p]), 0, par_beta[rho$coef.ind[[j]][,p]]) #TODO: ?ifelse
+  pred.fixedU  <- sapply(1:rho$ndim, function(j) {
+     b <- lapply(seq_along(rho$constraints),function(i)
+     rho$constraints[[i]][rho$inds.cat[[j]],,drop=F] %*%
+     par_beta[rho$nbeta.first[i] + seq_len(rho$npar.beta[i]) - 1])
+      rho$XcatU[[j]] %*% unlist(b) + rho$offset[[j]]
+    })
+  pred.fixedL  <- sapply(1:rho$ndim, function(j) {
+     b <- lapply(seq_along(rho$constraints),function(i)
+     rho$constraints[[i]][rho$inds.cat[[j]],,drop=F] %*%
+     par_beta[rho$nbeta.first[i] + seq_len(rho$npar.beta[i]) - 1])
+      rho$XcatL[[j]] %*% unlist(b) + rho$offset[[j]]
     })
-  })
-
-  pred.fixed <- sapply(1:rho$ndim, function(j) rowSums(rho$x[[j]] * beta[[j]]) + rho$offset[[j]])
-
   theta.lower <- sapply(1:rho$ndim, function(j) c(-rho$inf.value, theta[[j]])[rho$y[, j]])
   theta.upper <- sapply(1:rho$ndim, function(j) c(theta[[j]], rho$inf.value)[rho$y[, j]])
-  pred.lower <- (theta.lower - pred.fixed)/sigmas$sdVec
-  pred.upper <- (theta.upper - pred.fixed)/sigmas$sdVec
+  pred.lower <- (theta.lower - pred.fixedL)/sigmas$sdVec
+  pred.upper <- (theta.upper - pred.fixedU)/sigmas$sdVec
   list(U = pred.upper,
        L = pred.lower,
        corr_par = sigmas$rVec,
@@ -248,7 +252,7 @@ backtransf_sigmas <- function(R){
 # #' which specifies the ordering of the repeated measurements.
 # #' @export
 mvord_data <- function(data, index, y.names, x.names,
-                         y.levels = NULL, response.names = NULL) {
+                         y.levels = NULL, response.names = NULL, contrasts) {
   df <- list()
   if (any(duplicated(data[,index]))) stop("duplicated observation(s) for one index", call. = FALSE)
   index.levels <- levels(as.factor(data[, index[2]]))
@@ -306,7 +310,7 @@ mvord_data <- function(data, index, y.names, x.names,
     df$y <- df$y[-indallNA, ]
     df$x <- lapply(df$x, function(a) {a <- a[-indallNA, ]; a})
   }
-
+  attr(df, "contrasts") <- contrasts
   df
 }
 
@@ -331,43 +335,51 @@ is.offset <- function(expr) {
 
 get_constraints <- function(rho){
 if(is.list(rho$coef.constraints)){
+  ## check if nrow is sum(ncat_j - 1)
+  if(!(all(sapply(rho$coef.constraints, nrow) == rho$nthetas))) stop("The constraint matrices must have nrow equal to sum(ncat_j - 1).", call. = FALSE)
   if(is.null(names(rho$coef.constraints))) names(rho$coef.constraints) <- rho$coef.names
   if (!all(rho$coef.names %in% names(rho$coef.constraints))) stop("coef.constraints need to be specified for all covariates
                                                                     and intercept if included.", call. = FALSE)
   constraints <- rho$coef.constraints[match(rho$coef.names, names(rho$coef.constraints))]
-  constraints <- lapply(constraints, function(p) {
-    if (!(is.matrix(p))) p <-  as.matrix(p)
-    p
-  })
+  constraints <- lapply(constraints, as.matrix)
 } else{ #matrix to VGAM
-  constraints <- lapply(seq_along(rho$coef.names), function(p) {
-    tmp <- matrix(0, ncol = sum(!is.na(unique(rho$coef.constraints[, p]))), nrow = rho$ncats)
-    if(!is.na(rho$coef.constraints[1, p])) tmp[seq_len(rho$ncat[1]), 1] <- 1
+  constraints <- lapply(seq_len(NCOL(rho$coef.constraints)), function(p) {
+    tmp <- matrix(0,
+      ncol = sum(!is.na(unique(rho$coef.constraints[, p]))),
+      nrow = rho$nthetas)
+    if(!is.na(rho$coef.constraints[1, p])) tmp[seq_len(rho$ntheta[1]), 1] <- 1
     for (j in 2:nrow(rho$coef.constraints)){
       if (is.na(rho$coef.constraints[j, p])){
         tmp <- tmp
       } else if (rho$coef.constraints[j, p] %in% rho$coef.constraints[1:(j-1), p]){
-        tmp[rho$ncat.first.ind[j]:sum(rho$ncat[seq_len(j)]),
+        tmp[rho$ncat.first.ind[j]:sum(rho$ntheta[seq_len(j)]),
             which(rho$coef.constraints[j, p] %in% rho$coef.constraints[1:(j-1), p])] <- 1
       } else{
-        tmp[rho$ncat.first.ind[j]:sum(rho$ncat[seq_len(j)]),
+        tmp[rho$ncat.first.ind[j]:sum(rho$ntheta[seq_len(j)]),
             sum(!is.na(unique(rho$coef.constraints[1:(j-1),p]))) + 1] <- 1
       }
     }
     tmp
   })
+  constraints<-constraints[sapply(constraints,NCOL)!=0]
+  }
+  constraints <- lapply(seq_along(constraints), function(p) {
+    colnames(constraints[[p]]) <- paste(rho$coef.names[p], seq_len(NCOL(constraints[[p]])))
+    rownames(constraints[[p]]) <- unlist(lapply(1:rho$ndim, function(j)
+      get_labels_theta(rho, j)))
+    constraints[[p]]
+  })
   names(constraints) <- rho$coef.names
   constraints
 }
-}
 
-get_ind_coef <- function(rho){
+get_ind_coef <- function(constraints, rho){
   lapply(seq_len(rho$ndim), function(j){
   ind <- as.integer(rho$y[, rho$y.names[j]])
   sapply(seq_along(rho$coef.names), function(p) {
-    if(NCOL(rho$constraints[[p]]) == 0) tmp <- rep(NA, rho$ncat[j]) else{
+    if(NCOL(constraints[[p]]) == 0) tmp <- rep(NA, rho$ntheta[j]) else{
       #CHeCK if no 1 in row
-      tmp <- apply(rho$constraints[[p]][rho$ncat.first.ind[j]:sum(rho$ncat[seq_len(j)]), , drop = FALSE] == 1, 1,
+      tmp <- apply(constraints[[p]][rho$ncat.first.ind[j]:sum(rho$ntheta[seq_len(j)]), , drop = FALSE] == 1, 1,
                    function(x) {
                      y <- which(x)
                      if(length(y) > 0) y else NA
@@ -389,7 +401,41 @@ if (all(sapply(rho$offset, is.null))) {
       tmp2 <- c(rho$x[[j]] %*% tmp[j,])
       tmp2[is.na(tmp2)] <- 0
       tmp2
-    })} else  offset <- rep(list(0), rho$ndim)
+    }
+    )} else  offset <- lapply(1:rho$ndim, function(j) rep(0, rho$n))
     offset
 } else rho$offset
 }
+
+
+set_offset_up <- function(rho){
+if (all(sapply(rho$offset, is.null))) {
+  if (any(rho$coef.values != 0, na.rm = TRUE)){
+    tmp <- rho$coef.values
+    tmp[is.na(tmp)] <- 0
+    rho$offset <- lapply(1:rho$ndim, function(j){
+      tmp2 <- c(rho$x[[j]] %*% tmp[j,])
+      tmp2[is.na(tmp2)] <- 0
+      tmp2
+    })
+  } else {
+    rho$offset <- lapply(1:rho$ndim, function(j) rep(0, rho$n))
+  }
+}
+  if (!is.null(rho$coef.values)){
+  wh_fix <- which(colSums(is.na(rho$coef.values)) == 0)
+  if (length(wh_fix) != 0){
+  for (j in 1:rho$ndim) {
+     rho$x[[j]] <-  rho$x[[j]][, -wh_fix, drop = F]
+  }
+  }
+}
+  rho
+}
+
+#is.partial_prop_odds <- function(x, ncat, first.ind){
+#  inds <- lapply(seq_len(length(ncat)), function(j) seq_len(ncat[j]) + first.ind[j] - 1)
+#  sapply(seq_len(length(ncat)), function(j){
+#    if (all(apply(x[inds[[j]], ,drop = FALSE] == 1, 2, all) | apply(x[inds[[j]], ,drop = FALSE] == 0, 2, all))) FALSE else TRUE
+#  })
+#}
